@@ -1,5 +1,5 @@
 /*
- * $Id: data_lump_rpl.h,v 1.4 2002/09/19 12:23:52 jku Rel $
+ * $Id: data_lump_rpl.h,v 1.5.4.2 2003/12/18 22:46:42 bogdan Exp $
  *
  *
  * Copyright (C) 2001-2003 Fhg Fokus
@@ -24,6 +24,13 @@
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ * History:
+ * 2002-02-14 : created by bogdan
+ * 2003-09-11 : lump_rpl type added - LUMP_RPL_BODY & LUMP_RPL_HDR (bogdan)
+ * 2003-11-11 : build_lump_rpl merged into add_lump_rpl; type removed;
+ *              flags LUMP_RPL_BODY, LUMP_RPL_NODUP and LUMP_RPL_NOFREE
+ *              added (bogdan)
  */
 
 
@@ -33,16 +40,25 @@
 #include "parser/msg_parser.h"
 
 
+#define LUMP_RPL_HDR     (1<<1)
+#define LUMP_RPL_BODY    (1<<2)
+#define LUMP_RPL_NODUP   (1<<3)
+#define LUMP_RPL_NOFREE  (1<<4)
+#define LUMP_RPL_SHMEM   (1<<5)
+
 struct lump_rpl
 {
 	str text;
+	int flags;
 	struct lump_rpl* next;
 };
 
-struct lump_rpl* build_lump_rpl( char* , int );
-
-void add_lump_rpl(struct sip_msg * , struct lump_rpl* );
+struct lump_rpl* add_lump_rpl(struct sip_msg *, char *, int , int );
 
 void free_lump_rpl(struct lump_rpl* );
 
+void unlink_lump_rpl(struct sip_msg *, struct lump_rpl* );
+
+void del_nonshm_lump_rpl(  struct lump_rpl ** );
+
 #endif
