@@ -1,5 +1,5 @@
 /*
- * $Id: fifo_server.c,v 1.25 2002/10/22 20:37:29 andrei Exp $
+ * $Id: fifo_server.c,v 1.39.2.1.2.1 2003/12/03 12:15:04 andrei Exp $
  *
  *
  * Copyright (C) 2001-2003 Fhg Fokus
@@ -50,21 +50,33 @@
  *
  * File test/transaction.fifo illustrates example of use
  * of t_uac command (part of TM module).
+ *
+ * History:
+ * --------
+ *  2003-03-29  destroy pkg mem introduced (jiri)
+ *  2003-03-19  replaced all mallocs/frees w/ pkg_malloc/pkg_free (andrei)
+ *  2003-01-29  new built-in fifo commands: arg and pwd (jiri)
+ *  2003-10-07  fifo security fixes: permissions, always delete old fifo,
+ *               reply fifo checks -- added fifo_check (andrei)
  */
 
 
+#include <limits.h>
 #include <stdlib.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <errno.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <signal.h>
 #include <string.h>
 #include <time.h>
 #include <stdarg.h>
+#ifdef USE_TCP
+#include <sys/socket.h>
+#endif
+
 #include "dprint.h"
 #include "ut.h"
 #include "error.h"
@@ -77,8 +89,7 @@
 
 /* FIFO server vars */
 char *fifo=0; /* FIFO name */
-int fifo_mode=S_IRUSR | S_IWUSR | S_IRGRP | 
-	S_IWGRP | S_IROTH | S_IWOTH;
+int fifo_mode=S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP ;
 pid_t fifo_pid;
 /* file descriptors */
 static int fifo_read=0;
@@ -109,7 +120,7 @@ int register_fifo_cmd(fifo_cmd f, char *cmd_name, void *param)
 		LOG(L_ERR, "ERROR: register_fifo_cmd: attempt to register synonyms\n");
 		return E_BUG;
 	}
-	new_cmd=malloc(sizeof(struct fifo_command));
+	new_cmd=pkg_malloc(sizeof(struct fifo_command));
 	if (new_cmd==0) {
 		LOG(L_ERR, "ERROR: register_fifo_cmd: out of mem\n");
 		return E_OUT_OF_MEM;
@@ -126,6 +137,19 @@ int register_fifo_cmd(fifo_cmd f, char *cmd_name, void *param)
 	return 1;
 }
 
+void destroy_fifo()
+{
+	struct fifo_command *c, *foo;
+
+	c=cmd_list;
+
+	while(c) {
+		foo=c->next;
+		pkg_free(c);
+		c=foo;
+	}
+}
+
 
 int read_line( char *b, int max, FILE *stream, int *read )
 {
@@ -177,18 +201,6 @@ static void consume_request( FILE *stream )
 
 	while(!read_line(buffer, MAX_CONSUME_BUFFER, stream, &len));
 
-#ifdef _OBSOLETED
-	int eol_count;
-
-	eol_count=0;
-
-	/* each request must be terminated by two EoLs */
-	while(eol_count!=2) {
-		/* read until EoL is encountered */
-		while(!read_line(buffer, MAX_CONSUME_BUFFER, stream, &len));
-		eol_count=len==0?eol_count+1:1;
-	}
-#endif
 }
 
 int read_eol( FILE *stream )
@@ -217,7 +229,7 @@ int read_line_set(char *buf, int max_len, FILE *fifo, int *len)
 			return 0;
 		}
 		/* end encountered ... return */
-		if (line_len==0) {
+		if (line_len==0 || (line_len==1 && c[0]=='.' )) {
 			*len=set_len;
 			return 1;
 		}
@@ -231,6 +243,7 @@ int read_line_set(char *buf, int max_len, FILE *fifo, int *len)
 	}
 }
 
+
 /* read from input until line with only dot in it is encountered */
 int read_body(char *buf, int max_len, FILE *fifo, int *len)
 {
@@ -287,6 +300,60 @@ static char *trim_filename( char * file )
 	return new_fn;
 }
 
+
+
+/* reply fifo security checks:
+ * checks if fd is a fifo, is not hardlinked and it's not a softlink
+ * opened file descriptor + file name (for soft link check)
+ * returns 0 if ok, <0 if not */
+static int fifo_check(int fd, char* fname)
+{
+	struct stat fst;
+	struct stat lst;
+	
+	if (fstat(fd, &fst)<0){
+		LOG(L_ERR, "ERROR: fifo_check: fstat failed: %s\n",
+				strerror(errno));
+		return -1;
+	}
+	/* check if fifo */
+	if (!S_ISFIFO(fst.st_mode)){
+		LOG(L_ERR, "ERROR: fifo_check: %s is not a fifo\n", fname);
+		return -1;
+	}
+	/* check if hard-linked */
+	if (fst.st_nlink>1){
+		LOG(L_ERR, "ERROR: security: fifo_check: %s is hard-linked %d times\n",
+				fname, fst.st_nlink);
+		return -1;
+	}
+	
+	/* lstat to check for soft links */
+	if (lstat(fname, &lst)<0){
+		LOG(L_ERR, "ERROR: fifo_check: lstat failed: %s\n",
+				strerror(errno));
+		return -1;
+	}
+	if (S_ISLNK(lst.st_mode)){
+		LOG(L_ERR, "ERROR: security: fifo_check: %s is a soft link\n",
+				fname);
+		return -1;
+	}
+	/* if this is not a symbolic link, check to see if the inode didn't
+	 * change to avoid possible sym.link, rm sym.link & replace w/ fifo race
+	 */
+	if ((lst.st_dev!=fst.st_dev)||(lst.st_ino!=fst.st_ino)){
+		LOG(L_ERR, "ERROR: security: fifo_check: inode/dev number differ"
+				": %d %d (%s)\n",
+				 (int)fst.st_ino, (int)lst.st_ino, fname);
+		return -1;
+	}
+	/* success */
+	return 0;
+}
+
+
+
 /* tell FIFO client what happened via reply pipe */
 void fifo_reply( char *reply_fifo, char *reply_fmt, ... )
 {
@@ -355,6 +422,10 @@ tryagain:
 			pipe_name, strerror(errno));
 		return 0;
 	}
+	/* security checks: is this really a fifo?, is 
+	 * it hardlinked? is it a soft link? */
+	if (fifo_check(fifofd, pipe_name)<0) goto error;
+	
 	/* we want server blocking for big writes */
 	if ( (flags=fcntl(fifofd, F_GETFL, 0))<0) {
 		LOG(L_ERR, "ERROR: open_reply_pipe (%s): getfl failed: %s\n",
@@ -401,7 +472,7 @@ static void fifo_server(FILE *fifo_stream)
 			goto consume;
 		}
 		if (line_len==0) {
-			LOG(L_INFO, "INFO: fifo_server: command empty\n");
+			LOG(L_DBG, "INFO: fifo_server: command empty\n");
 			continue;
 		}
 		if (line_len<3) {
@@ -457,32 +528,64 @@ consume:
 int open_fifo_server()
 {
 	char *t;
+	struct stat filestat;
+	int n;
+#ifdef USE_TCP
+	int sockfd[2];
+#endif
 
 	if (fifo==NULL) {
-		DBG("TM: open_uac_fifo: no fifo will be opened\n");
+		DBG("DBG: open_fifo_server: no fifo will be opened\n");
 		/* everything is ok, we just do not want to start */
 		return 1;
 	}
 	if (strlen(fifo)==0) {
-		DBG("TM: open_uac_fifo: fifo disabled\n");
+		DBG("DBG: open_fifo_server: fifo disabled\n");
 		return 1;
 	}
-	DBG("TM: open_uac_fifo: opening fifo...\n");
-	if ((mkfifo(fifo, fifo_mode)<0) && (errno!=EEXIST)) {
-		LOG(L_ERR, "ERROR: open_fifo_server; can't create FIFO: "
-			"%s (mode=%d)\n",
-			strerror(errno), fifo_mode);
-		return -1;
-	} 
+	DBG("DBG: open_uac_fifo: opening fifo...\n");
+	n=stat(fifo, &filestat);
+	if (n==0){
+		/* FIFO exist, delete it (safer) */
+		if (unlink(fifo)<0){
+			LOG(L_ERR, "ERROR: open_fifo_server: cannot delete old fifo (%s):"
+					" %s\n", fifo, strerror(errno));
+			return -1;
+		}
+	}else if (n<0 && errno!=ENOENT){
+		LOG(L_DBG, "DEBUG: open_fifo_server: FIFO stat failed: %s\n",
+			strerror(errno));
+	}
+	/* create FIFO ... */
+		if ((mkfifo(fifo, fifo_mode)<0)) {
+			LOG(L_ERR, "ERROR: open_fifo_server; can't create FIFO: "
+					"%s (mode=%d)\n",
+					strerror(errno), fifo_mode);
+			return -1;
+		} 
+		DBG("DEBUG: FIFO created @ %s\n", fifo );
+		if ((chmod(fifo, fifo_mode)<0)) {
+			LOG(L_ERR, "ERROR: open_fifo_server; can't chmod FIFO: "
+					"%s (mode=%d)\n",
+					strerror(errno), fifo_mode);
+			return -1;
+		}
 	DBG("DEBUG: fifo %s opened, mode=%d\n", fifo, fifo_mode );
 	time(&up_since);
 	t=ctime(&up_since);
 	if (strlen(t)+1>=MAX_CTIME_LEN) {
 		LOG(L_ERR, "ERROR: open_fifo_server: "
-			"too long date %d\n", strlen(t));
+			"too long date %d\n", (int)strlen(t));
 		return -1;
 	}
 	memcpy(up_since_ctime,t,strlen(t)+1);
+#ifdef USE_TCP
+	if (socketpair(AF_UNIX, SOCK_STREAM, 0, sockfd)<0){
+			LOG(L_ERR, "ERROR: open_fifo_server: socketpair failed: %s\n",
+				strerror(errno));
+			return -1;
+	}
+#endif
 	process_no++;
 	fifo_pid=fork();
 	if (fifo_pid<0) {
@@ -495,7 +598,11 @@ int open_fifo_server()
 		/* call per-child module initialization too -- some
 		   FIFO commands may need it
 		*/
-		if (init_child(process_no) < 0 ) {
+#ifdef USE_TCP
+		close(sockfd[0]);
+		unix_tcp_sock=sockfd[1];
+#endif
+		if (init_child(PROC_FIFO) < 0 ) {
 			LOG(L_ERR, "ERROR: open_uac_fifo: init_child failed\n");
 			return -1;
 		}
@@ -520,6 +627,11 @@ int open_fifo_server()
 	/* dad process */
 	pt[process_no].pid=fifo_pid;
 	strncpy(pt[process_no].desc, "fifo server", MAX_PT_DESC );
+#ifdef USE_TCP
+	close(sockfd[1]);
+	pt[process_no].unix_sock=sockfd[0];
+	pt[process_no].idx=-1; /* this is not "tcp" process*/
+#endif
 	/* make sure the read fifo will not close */
 	fifo_write=open(fifo, O_WRONLY, 0);
 	if (fifo_write<0) {
@@ -535,10 +647,63 @@ static int print_version_cmd( FILE *stream, char *response_file )
 	if (response_file) {
 		fifo_reply(response_file, "200 ok\n" SERVER_HDR CRLF );
 	} else {
-		LOG(L_ERR, "ERROR: no file for print_version_cmd\n");
+		LOG(L_ERR, "ERROR: no file for %s\n", "print_version_cmd" );
 	}
 	return 1;
 }
+
+static int pwd_cmd( FILE *stream, char *response_file )
+{
+	char *cwd_buf;
+	int max_len;
+
+	if (!response_file) {
+		LOG(L_ERR, "ERROR: no file for %s\n", "pwd_cmd" );
+		return 1;
+	}
+
+	max_len=pathmax();
+	cwd_buf=pkg_malloc(max_len);
+	if (!cwd_buf) {
+		LOG(L_ERR, "ERROR: pwd_cmd: no cwd pkg mem\n");
+		fifo_reply(response_file, "500 no memory\n");
+		return 1;
+	}
+
+	if (getcwd(cwd_buf, max_len)) {
+		fifo_reply(response_file, "200 ok\n%s\n", cwd_buf );
+	} else {
+		fifo_reply(response_file, "500 getcwd failed\n" );
+	}
+
+	pkg_free(cwd_buf);
+	return 1;
+}
+
+static int arg_cmd( FILE *stream, char *response_file )
+{
+	FILE *reply_pipe;
+	int p;
+
+	if (response_file==0 || *response_file==0 ) {
+		 LOG(L_ERR, "ERROR: ps_fifo_cmd: null file\n");
+		return -1;
+	}
+	reply_pipe=open_reply_pipe(response_file);
+	if (reply_pipe==NULL) {
+		LOG(L_ERR, "ERROR: ps_fifo_cmd: opening reply pipe (%s) failed\n",
+			response_file );
+		return -1;
+	}
+
+	fputs( "200 ok\n", reply_pipe);
+	for (p=0; p<my_argc;p++) 
+			fprintf( reply_pipe, "%s\n", my_argv[p] );
+			
+	fclose(reply_pipe);
+	return 1;
+}
+
 	
 
 /* diagnostic and hello-world FIFO command */
@@ -584,6 +749,17 @@ static int uptime_fifo_cmd( FILE *stream, char *response_file )
 	return 1;
 }
 
+static int kill_fifo_cmd( FILE *stream, char *response_file )
+{
+	if (response_file==0 || *response_file==0 ) { 
+		LOG(L_ERR, "ERROR: uptime_fifo_cmd: null file\n");
+		return -1;
+	}
+	fifo_reply(response_file, "200 killing now..." );
+	kill(0, SIGTERM);
+	return 1;
+}
+
 static int which_fifo_cmd(FILE *stream, char *response_file )
 {
 	FILE *reply_pipe;
@@ -649,6 +825,14 @@ int register_core_fifo()
 		LOG(L_CRIT, "unable to register '%s' FIFO cmd\n", FIFO_VERSION);
 		return -1;
 	}
+	if (register_fifo_cmd(pwd_cmd, FIFO_PWD, 0)<0) {
+		LOG(L_CRIT, "unable to register '%s' FIFO cmd\n", FIFO_PWD);
+		return -1;
+	}
+	if (register_fifo_cmd(arg_cmd, FIFO_ARG, 0)<0) {
+		LOG(L_CRIT, "unable to register '%s' FIFO cmd\n", FIFO_ARG);
+		return -1;
+	}
 	if (register_fifo_cmd(which_fifo_cmd, FIFO_WHICH, 0)<0) {
 		LOG(L_CRIT, "unable to register '%s' FIFO cmd\n", FIFO_WHICH);
 		return -1;
@@ -657,5 +841,9 @@ int register_core_fifo()
 		LOG(L_CRIT, "unable to register '%s' FIFO cmd\n", FIFO_PS);
 		return -1;
 	}
+	if (register_fifo_cmd(kill_fifo_cmd, FIFO_KILL, 0)<0) {
+		LOG(L_CRIT, "unable to register '%s' FIFO cmd\n", FIFO_KILL);
+		return -1;
+	}
 	return 1;
 }
