@@ -1,5 +1,5 @@
 /* 
- * $Id: pike_funcs.h,v 1.10 2002/09/19 12:23:54 jku Rel $
+ * $Id: pike_funcs.h,v 1.11.6.1 2004/05/12 12:41:05 bogdan Exp $
  *
  *
  * Copyright (C) 2001-2003 Fhg Fokus
@@ -28,28 +28,15 @@
 
 
 #ifndef _PIKE_FUNCS_H
-#define PIKE_FUNCS_H
+#define _PIKE_FUNCS_H
 
 #include "../../parser/msg_parser.h"
-#include "ip_tree.h"
-#include "lock.h"
-#include "timer.h"
-#include "ip_tree.h"
+#include "../../locking.h"
 
 
-enum pike_locks {
-	TREE_LOCK,
-	TIMER_LOCK,
-	PIKE_NR_LOCKS
-};
-
-
-extern int                     time_unit;
-extern int                     max_value;
-extern int                     timeout;
-extern struct ip_node          *tree;
-extern pike_lock               *locks;
-extern struct pike_timer_head  *timer;
+//extern int                     time_unit;
+//extern int                     max_value;
+//extern gen_lock_t              *timer_lock;
 
 
 int  pike_check_req(struct sip_msg *msg, char *foo, char *bar);
