@@ -1,5 +1,5 @@
 /*
- * $Id: save.h,v 1.2 2002/09/19 12:23:54 jku Rel $
+ * $Id: save.h,v 1.3 2003/03/21 20:36:24 janakj Exp $
  *
  * Functions that process REGISTER message 
  * and store data in usrloc
@@ -26,6 +26,10 @@
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ * History:
+ * -------
+ * 2003-03-21  save_noreply added, provided by Maxim Sobolev <sobomax@portaone.com> (janakj)
  */
 
 
@@ -41,5 +45,10 @@
  */
 int save(struct sip_msg* _m, char* _t, char* _s);
 
+/*
+ * Process REGISTER request and save it's contacts, do not send any replies
+ */
+int save_noreply(struct sip_msg* _m, char* _t, char* _s);
+
 
 #endif /* SAVE_H */
