@@ -1,5 +1,5 @@
 /*
- * $Id: sms_funcs.c,v 1.44 2002/10/28 15:44:41 bogdan Exp $
+ * $Id: sms_funcs.c,v 1.60.4.1 2004/07/02 16:00:33 andrei Exp $
  *
  * Copyright (C) 2001-2003 Fhg Fokus
  *
@@ -25,6 +25,16 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+/*
+ * History:
+ * --------
+ * 2003-01-23: switched from t_uac to t_uac_dlg, adapted to new way of
+ *              parsing for Content-Type (bogdan)
+ * 2003-02-28: protocolization of t_uac_dlg completed (jiri)
+ * 2003-08-05: adapted to the new parse_content_type_hdr function (bogdan)
+ * 2003-09-11: updated to new build_lump_rpl() interface (bogdan)
+ * 2003-09-11: force parsing to hdr when extracting destination user (bogdan)
+ */
 
 #include <unistd.h>
 #include <errno.h>
@@ -36,8 +46,9 @@
 #include "../../mem/mem.h"
 #include "../../mem/shm_mem.h"
 #include "../../parser/parse_uri.h"
+#include "../../parser/parse_content.h"
+#include "../../parser/parse_from.h"
 #include "../../data_lump_rpl.h"
-#include "../im/im_funcs.h"
 #include "../tm/t_hooks.h"
 #include "../tm/uac.h"
 #include "sms_funcs.h"
@@ -58,7 +69,6 @@ int *queued_msgs;
 int use_contact;
 int sms_report_type;
 struct tm_binds tmb;
-struct im_binds imb;
 
 
 #define ERR_NUMBER_TEXT " is an invalid number! Please resend your SMS "\
@@ -103,7 +113,7 @@ struct im_binds imb;
 
 
 
-
+#if 0
 inline int add_contact(struct sip_msg* msg , str* user)
 {
 	struct lump_rpl *lump;
@@ -128,7 +138,7 @@ inline int add_contact(struct sip_msg* msg , str* user)
 	*(p++) = '>';
 	append_str( p, CRLF, CRLF_LEN);
 
-	lump = build_lump_rpl( buf , len );
+	lump = build_lump_rpl( buf , len , LUMP_RPL_HDR);
 	if(!lump) {
 		LOG(L_ERR,"ERROR:sms_add_contact: unable to build lump_rpl! \n");
 		pkg_free( buf );
@@ -139,7 +149,7 @@ inline int add_contact(struct sip_msg* msg , str* user)
 	pkg_free(buf);
 	return 1;
 }
-
+#endif
 
 
 
@@ -148,21 +158,40 @@ int push_on_network(struct sip_msg *msg, int net)
 	str    body;
 	struct sip_uri  uri;
 	struct sms_msg  *sms_messg;
-	struct to_body  from_parsed;
-	struct to_param *foo,*bar;
-	char   *p, *buf=0;
-	int    flag;
+	struct to_body  *from;
+	char   *p;
 	int    len;
+	int    mime;
 
-	if ( imb.im_extract_body(msg,&body)==-1 )
-	{
-		LOG(L_ERR,"ERROR:sms_push_on_net:cannot extract body from msg!\n");
-		goto error1;
+	/* get the message's body
+	 * anyhow we have to call this function, so let's do it at the beginning
+	 * to force the parsing of all the headers - like this we avoid separat
+	 * calls of parse_headers function for FROM, CONTENT_LENGTH, TO hdrs  */
+	body.s = get_body( msg );
+	if (body.s==0) {
+		LOG(L_ERR,"ERROR:sms_push_on_net: cannot extract body from msg!\n");
+		goto error;
 	}
 
-	if (!msg->from) {
-		LOG(L_ERR,"ERROR:sms_push_on_net: no FROM header found!\n");
-		goto error1;
+	/* content-length (if present) must be already parsed */
+	if (!msg->content_length) {
+		LOG(L_ERR,"ERROR:sms_push_on_net: no Content-Length header found!\n");
+		goto error;
+	}
+	body.len = get_content_length( msg );
+
+	/* parse the content-type header */
+	if ( (mime=parse_content_type_hdr(msg))<1 ) {
+		LOG(L_ERR,"ERROR:sms_push_on_net:cannot parse Content-Type header\n");
+		goto error;
+	}
+
+	/* check the content-type value */
+	if ( mime!=(TYPE_TEXT<<16)+SUBTYPE_PLAIN
+	&& mime!=(TYPE_MESSAGE<<16)+SUBTYPE_CPIM ) {
+		LOG(L_ERR,"ERROR:sms_push_on_net: invalid content-type for a "
+			"message request! type found=%d\n",mime);
+		goto error;
 	}
 
 	/* we try to get the user name (phone number) first from the RURI 
@@ -170,28 +199,22 @@ int push_on_network(struct sip_msg *msg, int net)
 	   if it's missing there (like in requests generated by MSN MESSENGER),
 	   we go for "to" header
 	*/
-	flag = 0;
-	DBG("DEBUG:sms_push_on_net: tring to get user from R_uri\n");
-	if ( parse_uri( msg->first_line.u.request.uri.s,
-	msg->first_line.u.request.uri.len ,&uri)||(flag=1)!=1||!uri.user.len )
+	DBG("DEBUG:sms_push_on_net: tring to get user from new_uri\n");
+	if ( !msg->new_uri.s||parse_uri( msg->new_uri.s,msg->new_uri.len,&uri)
+	|| !uri.user.len )
 	{
-		if (flag) free_uri(&uri);
-		flag = 0;
-		DBG("DEBUG:sms_push_on_net: tring to get user from new_uri\n");
-		if ( !msg->new_uri.s||parse_uri( msg->new_uri.s,msg->new_uri.len,&uri)
-		|| (flag=1)!=1 || !uri.user.len )
+		DBG("DEBUG:sms_push_on_net: tring to get user from R_uri\n");
+		if ( parse_uri( msg->first_line.u.request.uri.s,
+		msg->first_line.u.request.uri.len ,&uri)||!uri.user.len )
 		{
-			if (flag) free_uri(&uri);
-			flag = 0;
 			DBG("DEBUG:sms_push_on_net: tring to get user from To\n");
-			if (!msg->to || !get_to(msg) ||
-			parse_uri( get_to(msg)->uri.s, get_to(msg)->uri.len, &uri)
-			|| (flag=1)!=1 ||!uri.user.len)
+			if ( (!msg->to&&((parse_headers(msg,HDR_TO,0)==-1) || !msg->to)) ||
+			parse_uri( get_to(msg)->uri.s, get_to(msg)->uri.len, &uri)==-1
+			|| !uri.user.len)
 			{
-				if (flag) free_uri(&uri);
 				LOG(L_ERR,"ERROR:sms_push_on_net: unable to extract user"
 					" name from RURI and To header!\n");
-				goto error1;
+				goto error;
 			}
 		}
 	}
@@ -204,36 +227,25 @@ int push_on_network(struct sip_msg *msg, int net)
 	}
 
 	/* parsing from header */
-	memset(&from_parsed,0,sizeof(from_parsed));
-	p = translate_pointer(msg->orig,msg->buf,msg->from->body.s);
-	buf = (char*)pkg_malloc(msg->from->body.len+1);
-	if (!buf) {
-		LOG(L_ERR,"ERROR:sms_push_on_net: no free pkg memory\n");
+	if ( parse_from_header( msg )==-1 ) {
+		LOG(L_ERR,"ERROR:sms_push_on_net: cannot get FROM header\n");
 		goto error;
 	}
-	memcpy(buf,p,msg->from->body.len+1);
-	parse_to(buf,buf+msg->from->body.len+1,&from_parsed);
-	if (from_parsed.error!=PARSE_OK ) {
-		LOG(L_ERR,"ERROR:sms_push_on_net: cannot parse from header\n");
-		goto error;
-	}
-	/* we are not intrested in from param-> le's free them now*/
-	for(foo=from_parsed.param_lst ; foo ; foo=bar){
-		bar = foo->next;
-		pkg_free(foo);
-	}
+	from = (struct to_body*)msg->from->parsed;
 
+#if 0
 	/* adds contact header into reply */
 	if (add_contact(msg,&(uri.user))==-1) {
 		LOG(L_ERR,"ERROR:sms_push_on_net:can't build contact for reply\n");
 		goto error;
 	}
+#endif
 
 	/*-------------BUILD AND FILL THE SMS_MSG STRUCTURE --------------------*/
 	/* computes the amount of memory needed */
-	len = SMS_HDR_BF_ADDR_LEN + from_parsed.uri.len
+	len = SMS_HDR_BF_ADDR_LEN + from->uri.len
 		+ SMS_HDR_AF_ADDR_LEN + body.len + SMS_FOOTER_LEN /*text to send*/
-		+ from_parsed.uri.len /* from */
+		+ from->uri.len /* from */
 		+ uri.user.len-1 /* to user (wihout '+') */
 		+ sizeof(struct sms_msg) ; /* the sms_msg structure */
 	/* allocs a new sms_msg structure in shared memory */
@@ -245,9 +257,9 @@ int push_on_network(struct sip_msg *msg, int net)
 	p = (char*)sms_messg + sizeof(struct sms_msg);
 
 	/* copy "from" into sms struct */
-	sms_messg->from.len = from_parsed.uri.len;
+	sms_messg->from.len = from->uri.len;
 	sms_messg->from.s = p;
-	append_str(p,from_parsed.uri.s,from_parsed.uri.len);
+	append_str(p,from->uri.s,from->uri.len);
 
 	/* copy "to.user" - we have to strip out the '+' */
 	sms_messg->to.len = uri.user.len-1;
@@ -278,13 +290,8 @@ int push_on_network(struct sip_msg *msg, int net)
 		goto error;
 	}
 
-	free_uri(&uri);
-	if (buf) pkg_free(buf);
 	return 1;
-error:
-	free_uri(&uri);
-	if (buf) pkg_free(buf);
-error1:
+ error:
 	return -1;
 }
 
@@ -336,10 +343,16 @@ int send_sip_msg_request(str *to, str *from_user, str *body)
 		append_str(p,">"CRLF,1+CRLF_LEN);
 	}
 
-	/** sending SIP MSG using IM - deprecated **/
-	/*foo = im_send_message(to, to, &from, &contact, body); */
-	/** sending with TM **/
-	foo = tmb.t_uac( &msg_type, to, &hdrs, body, &from, 0, 0, 0);
+	/* sending the request */
+	foo = tmb.t_request( &msg_type,   /* request type */
+			0,                        /* Request-URI */
+			to,                       /* To */
+			&from,                    /* From */
+			&hdrs,                    /* Additional headers including CRLF */
+			body,                     /* Message body */
+			0,                        /* Callback function */
+			0                         /* Callback parameter */
+		);
 	if (from.s) pkg_free(from.s);
 	if (hdrs.s) pkg_free(hdrs.s);
 	return foo;
@@ -502,8 +515,6 @@ int send_as_sms(struct sms_msg *sms_messg, struct modem *mdm)
 	}
 	return 1;
 error:
-	if (!(--(sms_messg->ref)))
-		shm_free(sms_messg);
 	if (ret_code==-1)
 		/* bad number */
 		send_error(sms_messg, sms_messg->to.s, sms_messg->to.len,
@@ -514,6 +525,9 @@ error:
 			text.s+SMS_HDR_BF_ADDR_LEN+sms_messg->from.len+SMS_HDR_AF_ADDR_LEN,
 			text.len-SMS_FOOTER_LEN-SMS_HDR_BF_ADDR_LEN-sms_messg->from.len-
 			SMS_HDR_AF_ADDR_LEN );
+
+	if (!(--(sms_messg->ref)))
+		shm_free(sms_messg);
 	return -1;
 }
 
@@ -608,9 +622,9 @@ int send_sms_as_sip( struct incame_sms *sms )
 				sip_addr.len = p-sip_addr.s;
 			} else {
 				/* parse to the next word */
-				DBG("*** Skipping word len=%d\n",sms->userdatalength);
+				/*DBG("*** Skipping word len=%d\n",sms->userdatalength);*/
 				while(p<sms->ascii+sms->userdatalength&&no_sip_addr_begin(*p)){
-					DBG("**** p=%c\n",*(p++));
+					p++;
 				}
 				p++;
 				if (p+9>=sms->ascii+sms->userdatalength) {
@@ -618,7 +632,7 @@ int send_sms_as_sip( struct incame_sms *sms )
 						"address start in sms body [%s]!\n",sms->ascii);
 					goto error;
 				}
-				DBG("*** Done\n");
+				/*DBG("*** Done\n");*/
 			}
 		}while (!sip_addr.len);
 	}
@@ -725,12 +739,11 @@ void modem_process(struct modem *mdm)
 	int counter;
 	int dont_wait;
 	int empty_pipe;
-	int last_smsc_index;
 	int cpms_unsuported;
 	int max_mem=0, used_mem=0;
 
+	sms_messg = 0;
 	cpms_unsuported = 0;
-	last_smsc_index = -1;
 
 	/* let's open/init the modem */
 	DBG("DEBUG:modem_process: openning modem\n");
@@ -749,6 +762,7 @@ void modem_process(struct modem *mdm)
 		used_mem = max_mem = 10;
 		cpms_unsuported = 1;
 	}
+	DBG("DEBUG:modem_process: modem maximum memory is %d\n",max_mem);
 
 	set_gettime_function();
 
@@ -760,8 +774,6 @@ void modem_process(struct modem *mdm)
 			counter = 0;
 			empty_pipe = 0;
 			net = &(networks[mdm->net_list[i]]);
-			DBG("DEBUG:modem_process: %s processing sms for net %s \n",
-				mdm->device, net->name);
 			/*getting msgs from pipe*/
 			while( counter<net->max_sms_per_call && !empty_pipe )
 			{
@@ -772,27 +784,24 @@ void modem_process(struct modem *mdm)
 					if (len>=0)
 						LOG(L_ERR,"ERROR:modem_process: truncated message"
 						" read from pipe! -> discarted\n");
-					else if (errno==EAGAIN) {
-						DBG("DEBUG:modem_process: out pipe emptied!! \n");
+					else if (errno==EAGAIN)
 						empty_pipe = 1;
-					}
+					else
+						LOG(L_ERR,"ERROR:modem_process: pipe reding failed: "
+							" : %s\n",strerror(errno));
+					sleep(1);
 					counter++;
 					continue;
 				}
 				(*queued_msgs)--;
 
-				/*sets the apropriat sms center*/
-				if (last_smsc_index!=mdm->net_list[i]) {
-					setsmsc(mdm,net->smsc);
-					last_smsc_index = mdm->net_list[i];
-				}
-
 				/* compute and send the sms */
-				DBG("DEBUG:modem_process: processing sms: \n\tTo:[%.*s] "
-					"\n\tBody=<%d>[%.*s]\n",sms_messg->to.len,sms_messg->to.s,
+				DBG("DEBUG:modem_process: %s processing sms for net %s:"
+					" \n\tTo:[%.*s]\n\tBody=<%d>[%.*s]\n",
+					mdm->device, net->name,
+					sms_messg->to.len,sms_messg->to.s,
 					sms_messg->text.len,sms_messg->text.len,sms_messg->text.s);
-				if ( send_as_sms( sms_messg , mdm)==-1 )
-					last_smsc_index = -1;
+				send_as_sms( sms_messg , mdm);
 
 				counter++;
 				/* if I reached the limit -> set not to wait */
@@ -807,11 +816,11 @@ void modem_process(struct modem *mdm)
 				LOG(L_ERR,"ERROR:modem_process: CPMS command failed!"
 					" cannot get used mem -> using 10\n");
 				used_mem = 10;
-				last_smsc_index = -1;
 			}
 
 		/* if any, let's get them */
 		if (used_mem)
+			DBG("DEBUG:modem_process: %d new SMS on modem\n",used_mem);
 			for(i=1,k=1;k<=used_mem && i<=max_mem;i++) {
 				if (getsms(&sms,mdm,i)!=-1) {
 					k++;
