@@ -1,7 +1,7 @@
-/*
- * $Id: case_to.h,v 1.3 2002/09/19 12:23:55 jku Rel $
+/* 
+ * $Id: case_to.h,v 1.6 2003/02/28 14:12:26 jiri Exp $ 
  *
- * To header field parser macros
+ * To Header Field Name Parsing Macros
  *
  * Copyright (C) 2001-2003 Fhg Fokus
  *
@@ -25,17 +25,22 @@
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ * History:
+ * ---------
+ * 2003-02-28 scratchpad compatibility abandoned (jiri)
+ * 2003-01-27 next baby-step to removing ZT - PRESERVE_ZT (jiri)
  */
 
 
 #ifndef CASE_TO_H
 #define CASE_TO_H
 
+#include "../comp_defs.h"
 
-#define To12_CASE           \
+#define to12_CASE           \
         hdr->type = HDR_TO; \
         hdr->name.len = 2;  \
-        *(p + 2) = '\0';    \
         return (p + 4);
 
 
