@@ -1,7 +1,7 @@
-/*
- * $Id: case_via.h,v 1.3 2002/09/19 12:23:55 jku Rel $
+/* 
+ * $Id: case_via.h,v 1.6 2003/02/28 14:12:26 jiri Exp $ 
  *
- * Via header field parser macros
+ * Via Header Field Name Parsing Macros
  *
  * Copyright (C) 2001-2003 Fhg Fokus
  *
@@ -25,21 +25,24 @@
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ * History:
+ * --------
+ * 2003-02-28 scratchpad compatibility abandoned (jiri)
  */
 
 
 #ifndef CASE_VIA_H
 #define CASE_VIA_H
 
+#include "../comp_defs.h"
 
-#define Via1_CASE            \
+#define via1_CASE            \
         hdr->type = HDR_VIA; \
         hdr->name.len = 3;   \
-        *(p + 3) = '\0';     \
         return (p + 4)        
 
-
-#define Via2_CASE            \
+#define via2_CASE            \
         hdr->type = HDR_VIA; \
         p += 4;              \
         goto dc_end
