@@ -1,5 +1,5 @@
 /*
- * $Id: parse_via.c,v 1.7 2002/09/19 12:23:55 jku Rel $ 
+ * $Id: parse_via.c,v 1.20 2003/07/09 17:59:50 andrei Exp $ 
  *
  * via parsing automaton
  * 
@@ -29,23 +29,32 @@
  */
 
 
-/* parsing:           compact form:
- */
 
 /* 
- * still TODO/test:
- *  x parse next via
- *  - return a list of header structs
- *  - return list of params
+ *  2003-01-21  added rport parsing code, contributed by
+ *               Maxim Sobolev  <sobomax@FreeBSD.org>
+ *  2003-01-23  added extra via param parsing code (i=...), used
+ *               by tcp to identify the sending socket, by andrei
+ *  2003-01-23  fixed rport parsing code to accept rport w/o any value,
+ *               by andrei
+ *  2003-01-27  modified parse_via to set new via_param->start member and
+ *               via->params.s (andrei)
+ *  2003-01-28  zero-terminations replaced with VIA_ZT (jiri)
+ *  2003-02-28  scratchpad compatibility abandoned (jiri)
+ *  2003-04-26  ZSW (jiri)
+ *  2003-06-23  fixed  parse_via_param [op].* param. parsing bug (andrei)
+ *  2003-07-02  added support for TLS parsing in via (andrei)
  */
 
 
 
+#include "../comp_defs.h"
 #include <stdlib.h>
 #include <string.h>
 #include "../dprint.h"
 #include "../ut.h"
 #include "../mem/mem.h"
+#include "../ip_addr.h"
 #include "parse_via.h"
 #include "parse_def.h"
 
@@ -58,7 +67,7 @@ enum {
 	L_PARAM, F_PARAM, P_PARAM,
 	L_VIA, F_VIA,
 	F_COMMENT, P_COMMENT,
-	F_IP6HOST, P_IP6HOST,
+	F_IP6HOST, P_IP6HOST, 
 	F_CRLF,
 	F_LF,
 	F_CR,
@@ -72,13 +81,16 @@ enum {
 	SIP1, SIP2, FIN_SIP,
 	L_VER, F_VER,
 	VER1, VER2, FIN_VER,
-	L_PROTO, F_PROTO, P_PROTO
+	UDP1, UDP2, FIN_UDP,
+	TCP_TLS1, TCP2, FIN_TCP,
+	          TLS2, FIN_TLS,
+	L_PROTO, F_PROTO
 };
 
 
 /* param related states
- * WARNING: keep the FIN*, GEN_PARAM & PARAM_ERROR in sync w/ PARAM_* from
- * msg_parser.h !*/
+ * WARNING: keep in sync with parse_via.h, PARAM_HIDDEN, ...
+ */
 enum {	
 	L_VALUE = 200, F_VALUE, P_VALUE, P_STRING,
 	HIDDEN1, HIDDEN2, HIDDEN3, HIDDEN4, HIDDEN5,
@@ -87,8 +99,10 @@ enum {
 	MADDR1, MADDR2, MADDR3, MADDR4,
 	RECEIVED1, RECEIVED2, RECEIVED3, RECEIVED4, RECEIVED5, RECEIVED6,
 	RECEIVED7,
+	RPORT1, RPORT2, RPORT3,
 	     /* fin states (227-...)*/
 	FIN_HIDDEN = 230, FIN_TTL, FIN_BRANCH, FIN_MADDR, FIN_RECEIVED,
+	FIN_RPORT, FIN_I
 	     /*GEN_PARAM,
 	       PARAM_ERROR*/ /* declared in msg_parser.h*/
 };
@@ -98,13 +112,17 @@ enum {
  * state=F_{LF,CR,CRLF}!
  * output state = L_PARAM or F_PARAM or END_OF_HEADER
  * (and saved_state= last state); everything else => error 
+ * WARNING: param->start must be filled before, it's used in param->size 
+ * computation.
  */
-static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate, 
-				    int* psaved_state, struct via_param* param)
+static /*inline*/ char* parse_via_param(char* p, char* end,
+										unsigned char* pstate, 
+				    					unsigned char* psaved_state,
+										struct via_param* param)
 {
 	char* tmp;
-	register int state;
-	int saved_state;
+	register unsigned char state;
+	unsigned char saved_state;
 
 	state=*pstate;
 	saved_state=*psaved_state;
@@ -116,7 +134,6 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 			case '\t':
 				switch(state){
 					case FIN_HIDDEN:
-						*tmp=0;
 						param->type=state;
 						param->name.len=tmp-param->name.s;
 						state=L_PARAM;
@@ -125,7 +142,8 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 					case FIN_TTL:
 					case FIN_MADDR:
 					case FIN_RECEIVED:
-						*tmp=0;
+					case FIN_RPORT:
+					case FIN_I:
 						param->type=state;
 						param->name.len=tmp-param->name.s;
 						state=L_VALUE;
@@ -139,7 +157,6 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 						break;
 					case GEN_PARAM:
 					default:
-						*tmp=0;
 						param->type=GEN_PARAM;
 						param->name.len=tmp-param->name.s;
 						state=L_VALUE;
@@ -150,9 +167,9 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 			case '\n':
 				switch(state){
 					case FIN_HIDDEN:
-						*tmp=0;
 						param->type=state;
 						param->name.len=tmp-param->name.s;
+						param->size=tmp-param->start; 
 						saved_state=L_PARAM;
 						state=F_LF;
 						goto endofparam;
@@ -160,9 +177,11 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 					case FIN_TTL:
 					case FIN_MADDR:
 					case FIN_RECEIVED:
-						*tmp=0;
+					case FIN_I:
+					case FIN_RPORT:
 						param->type=state;
 						param->name.len=tmp-param->name.s;
+						param->size=tmp-param->start; 
 						saved_state=L_VALUE;
 						state=F_LF;
 						goto find_value;
@@ -179,10 +198,10 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 						break;
 					case GEN_PARAM:
 					default:
-						*tmp=0;
 						param->type=GEN_PARAM;
 						saved_state=L_VALUE;
 						param->name.len=tmp-param->name.s;
+						param->size=tmp-param->start; 
 						state=F_LF;
 						goto find_value;
 				}
@@ -190,9 +209,9 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 			case '\r':
 				switch(state){
 					case FIN_HIDDEN:
-						*tmp=0;
 						param->type=state;
 						param->name.len=tmp-param->name.s;
+						param->size=tmp-param->start; 
 						saved_state=L_PARAM;
 						state=F_CR;
 						goto endofparam;
@@ -200,9 +219,11 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 					case FIN_TTL:
 					case FIN_MADDR:
 					case FIN_RECEIVED:
-						*tmp=0;
+					case FIN_I:
+					case FIN_RPORT:
 						param->type=state;
 						param->name.len=tmp-param->name.s;
+						param->size=tmp-param->start; 
 						saved_state=L_VALUE;
 						state=F_CR;
 						goto find_value;
@@ -216,9 +237,9 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 						goto end_via;
 					case GEN_PARAM:
 					default:
-						*tmp=0;
 						param->type=GEN_PARAM;
 						param->name.len=tmp-param->name.s;
+						param->size=tmp-param->start; 
 						saved_state=L_VALUE;
 						state=F_CR;
 						goto find_value;
@@ -231,7 +252,8 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 					case FIN_TTL:
 					case FIN_MADDR:
 					case FIN_RECEIVED:
-						*tmp=0;
+					case FIN_RPORT:
+					case FIN_I:
 						param->type=state;
 						param->name.len=tmp-param->name.s;
 						state=F_VALUE;
@@ -248,7 +270,6 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 						goto end_via;
 					case GEN_PARAM:
 					default:
-						*tmp=0;
 						param->type=GEN_PARAM;
 						param->name.len=tmp-param->name.s;
 						state=F_VALUE;
@@ -258,7 +279,7 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 			case ';':
 				switch(state){
 					case FIN_HIDDEN:
-						*tmp=0;
+					case FIN_RPORT: /* rport can appear w/o a value */
 						param->type=state;
 						param->name.len=tmp-param->name.s;
 						state=F_PARAM;
@@ -267,6 +288,7 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 					case FIN_MADDR:
 					case FIN_TTL:
 					case FIN_RECEIVED:
+					case FIN_I:
 						LOG(L_ERR, "ERROR: parse_via: invalid char <%c> in"
 								" state %d\n", *tmp, state);
 						goto error;
@@ -277,7 +299,6 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 						goto end_via;
 					case GEN_PARAM:
 					default:
-						*tmp=0;
 						param->type=GEN_PARAM;
 						param->name.len=tmp-param->name.s;
 						state=F_PARAM;
@@ -287,7 +308,7 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 			case ',':
 				switch(state){
 					case FIN_HIDDEN:
-						*tmp=0;
+					case FIN_RPORT:
 						param->type=state;
 						param->name.len=tmp-param->name.s;
 						state=F_VIA;
@@ -296,6 +317,7 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 					case FIN_MADDR:
 					case FIN_TTL:
 					case FIN_RECEIVED:
+					case FIN_I:
 						LOG(L_ERR, "ERROR: parse_via_param: new via found" 
 								"(',') when '=' expected (state %d=)\n",
 								state);
@@ -307,7 +329,6 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 						goto end_via;
 					case GEN_PARAM:
 					default:
-						*tmp=0;
 						param->type=GEN_PARAM;
 						param->name.len=tmp-param->name.s;
 						state=F_VIA;
@@ -341,7 +362,7 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 			case 'I':
 				switch(state){
 					case F_PARAM:
-						state=GEN_PARAM;
+						state=FIN_I;
 						param->name.s=tmp;
 						break;
 					case HIDDEN1:
@@ -458,6 +479,9 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 					case TTL1:
 						state=TTL2;
 						break;
+					case RPORT3:
+						state=FIN_RPORT;
+						break;
 					case GEN_PARAM:
 						break;
 					case F_CR:
@@ -545,6 +569,9 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 					case BRANCH1:
 						state=BRANCH2;
 						break;
+					case RPORT2:
+						state=RPORT3;
+						break;
 					case GEN_PARAM:
 						break;
 					case F_CR:
@@ -619,14 +646,51 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 						state=GEN_PARAM;
 				}
 				break;
-
+			case 'p':
+			case 'P':
+				switch(state){
+					case F_PARAM:
+						state=GEN_PARAM;
+						param->name.s=tmp;
+						break;
+					case RECEIVED1:
+						state=RPORT1;
+						break;
+					case F_CR:
+					case F_LF:
+					case F_CRLF:
+						state=END_OF_HEADER;
+						goto end_via;
+					default:
+						state=GEN_PARAM;
+				}
+				break;
+			case 'o':
+			case 'O':
+				switch(state){
+					case F_PARAM:
+						state=GEN_PARAM;
+						param->name.s=tmp;
+						break;
+					case RPORT1:
+						state=RPORT2;
+						break;
+					case F_CR:
+					case F_LF:
+					case F_CRLF:
+						state=END_OF_HEADER;
+						goto end_via;
+					default:
+						state=GEN_PARAM;
+				}
+				break;
 			default:
 				switch(state){
 					case F_PARAM:
 						state=GEN_PARAM;
 						param->name.s=tmp;
 						break;
-					case  GEN_PARAM:
+					case GEN_PARAM:
 						break;
 					case F_CR:
 					case F_LF:
@@ -655,7 +719,6 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 					case F_VALUE: /*eat space*/
 						break; 
 					case P_VALUE:
-						*tmp=0;
 						state=L_PARAM;
 						param->value.len=tmp-param->value.s;
 						goto endofvalue;
@@ -678,10 +741,10 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 					case F_VALUE: /*eat space*/
 					case P_STRING:
 						saved_state=state;
+						param->size=tmp-param->start;
 						state=F_LF;
 						break;
 					case P_VALUE:
-						*tmp=0;
 						saved_state=L_PARAM;
 						state=F_LF;
 						param->value.len=tmp-param->value.s;
@@ -705,10 +768,10 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 					case F_VALUE: /*eat space*/
 					case P_STRING:
 						saved_state=state;
+						param->size=tmp-param->start;
 						state=F_CR;
 						break;
 					case P_VALUE:
-						*tmp=0;
 						param->value.len=tmp-param->value.s;
 						saved_state=L_PARAM;
 						state=F_CR;
@@ -746,10 +809,13 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 			case ';':
 				switch(state){
 					case P_VALUE:
-						*tmp=0;
 						param->value.len=tmp-param->value.s;
 						state=F_PARAM;
 						goto endofvalue;
+					case F_VALUE:
+						param->value.len=0;
+						state=F_PARAM;
+						goto endofvalue;
 					case P_STRING:
 						break; /* what to do? */
 					case F_LF:
@@ -757,6 +823,14 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 					case F_CRLF:
 						state=END_OF_HEADER;
 						goto end_via;
+					case L_VALUE:
+						if (param->type==FIN_RPORT){
+							param->value.len=0;
+							param->value.s=0; /* null value */
+							state=F_PARAM;
+							goto endofvalue;
+						};
+						/* no break */
 					default:
 						LOG(L_ERR, "ERROR: parse_via: invalid char <%c>"
 								" in state %d\n", *tmp, state);
@@ -766,16 +840,23 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 			case ',':
 				switch(state){
 					case P_VALUE:
-						*tmp=0;
 						param->value.len=tmp-param->value.s;
 						state=F_VIA;
 						goto endofvalue;
 					case P_STRING:
-						case F_LF:
+					case F_LF:
 					case F_CR:
 					case F_CRLF:
 						state=END_OF_HEADER;
 						goto end_via;
+					case L_VALUE:
+						if (param->type==FIN_RPORT){
+							param->value.len=0;
+							param->value.s=0; /* null value */
+							state=F_VIA;
+							goto endofvalue;
+						};
+						/* no break */
 					default:
 						LOG(L_ERR, "ERROR: parse_via: invalid char <%c>"
 								" in state %d\n", *tmp, state);
@@ -789,7 +870,6 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 						param->value.s=tmp+1;
 						break;
 					case P_STRING:
-						*tmp=0;
 						state=L_PARAM;
 						param->value.len=tmp-param->value.s;
 						goto endofvalue;
@@ -833,25 +913,31 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 	
  endofparam:
  endofvalue:
-	param->size=tmp-p;
+	param->size=tmp-param->start;
+normal_exit:
 	*pstate=state;
 	*psaved_state=saved_state;
-	DBG("Found param type %d, <%s> = <%s>; state=%d\n", param->type,
-			param->name.s, param->value.s, state);
+	DBG("Found param type %d, <%.*s> = <%.*s>; state=%d\n", param->type,
+			param->name.len, ZSW(param->name.s), 
+			(param->value.len?param->value.len:3),
+			(param->value.len?param->value.s:"n/a"), state);
 	return tmp;
 	
  end_via:
 	     /* if we are here we found an "unexpected" end of via
-	      *  (cr/lf). This is valid only if the param type is GEN_PARAM*/
-	if (param->type==GEN_PARAM){
+	      *  (cr/lf). This is valid only if the param type is GEN_PARAM or
+		  *  RPORT (the only ones which can miss the value; HIDDEN is a 
+		  *  special case )*/
+	if ((param->type==GEN_PARAM)||(param->type==PARAM_RPORT)){
 		saved_state=L_PARAM; /* change the saved_state, we have an unknown
 		                        param. w/o a value */
-		goto endofparam;
+		/* param->size should be computed before */
+		goto normal_exit;
 	}
 	*pstate=state;
 	*psaved_state=saved_state;
-	DBG("Error on  param type %d, <%s>, state=%d, saved_state=%d\n",
-		param->type, param->name.s, state, saved_state);
+	DBG("Error on  param type %d, <%.*s>, state=%d, saved_state=%d\n",
+		param->type, param->name.len, ZSW(param->name.s), state, saved_state);
 
  error:
 	LOG(L_ERR, "error: parse_via_param\n");
@@ -866,8 +952,9 @@ static /*inline*/ char* parse_via_param(char* p, char* end, int* pstate,
 char* parse_via(char* buffer, char* end, struct via_body *vb)
 {
 	char* tmp;
-	int state;
-	int saved_state;
+	char* param_start;
+	unsigned char state;
+	unsigned char saved_state;
 	int c_nest;
 	int err;
 
@@ -877,6 +964,7 @@ parse_again:
 	vb->error=PARSE_ERROR;
 	/* parse start of via ( SIP/2.0/UDP    )*/
 	state=F_SIP;
+	param_start=0;
 	for(tmp=buffer;tmp<end;tmp++){
 		switch(*tmp){
 			case ' ':
@@ -888,18 +976,28 @@ parse_again:
 					case F_VER:
 					case F_PROTO:
 						break;
-					case P_PROTO:
-						*tmp=0;  /* finished proto parsing */
+					case FIN_UDP:
 						vb->transport.len=tmp-vb->transport.s;
+						vb->proto=PROTO_UDP;
+						state=F_HOST; /* start looking for host*/
+						goto main_via;
+					case FIN_TCP:
+						/* finished proto parsing */
+						vb->transport.len=tmp-vb->transport.s;
+						vb->proto=PROTO_TCP;
+						state=F_HOST; /* start looking for host*/
+						goto main_via;
+					case FIN_TLS:
+						/* finished proto parsing */
+						vb->transport.len=tmp-vb->transport.s;
+						vb->proto=PROTO_TLS;
 						state=F_HOST; /* start looking for host*/
 						goto main_via;
 					case FIN_SIP:
-						*tmp=0;
 						vb->name.len=tmp-vb->name.s;
 						state=L_VER;
 						break;
 					case FIN_VER:
-						*tmp=0;
 						vb->version.len=tmp-vb->version.s;
 						state=L_PROTO;
 						break;
@@ -924,20 +1022,30 @@ parse_again:
 						saved_state=state;
 						state=F_LF;
 						break;
-					case P_PROTO:
-						*tmp=0;
+					case FIN_UDP:
 						vb->transport.len=tmp-vb->transport.s;
+						vb->proto=PROTO_UDP;
+						state=F_LF;
+						saved_state=F_HOST; /* start looking for host*/
+						goto main_via;
+					case FIN_TCP:
+						vb->transport.len=tmp-vb->transport.s;
+						vb->proto=PROTO_TCP;
+						state=F_LF;
+						saved_state=F_HOST; /* start looking for host*/
+						goto main_via;
+					case FIN_TLS:
+						vb->transport.len=tmp-vb->transport.s;
+						vb->proto=PROTO_TLS;
 						state=F_LF;
 						saved_state=F_HOST; /* start looking for host*/
 						goto main_via;
 					case FIN_SIP:
-						*tmp=0;
 						vb->name.len=tmp-vb->name.s;
 						state=F_LF;
 						saved_state=L_VER;
 						break;
 					case FIN_VER:
-						*tmp=0;
 						vb->version.len=tmp-vb->version.s;
 						state=F_LF;
 						saved_state=L_PROTO;
@@ -965,20 +1073,30 @@ parse_again:
 						saved_state=state;
 						state=F_CR;
 						break;
-					case P_PROTO:
-						*tmp=0;
+					case FIN_UDP:
 						vb->transport.len=tmp-vb->transport.s;
+						vb->proto=PROTO_UDP;
+						state=F_CR;
+						saved_state=F_HOST;
+						goto main_via;
+					case FIN_TCP:
+						vb->transport.len=tmp-vb->transport.s;
+						vb->proto=PROTO_TCP;
+						state=F_CR;
+						saved_state=F_HOST;
+						goto main_via;
+					case FIN_TLS:
+						vb->transport.len=tmp-vb->transport.s;
+						vb->proto=PROTO_TLS;
 						state=F_CR;
 						saved_state=F_HOST;
 						goto main_via;
 					case FIN_SIP:
-						*tmp=0;
 						vb->name.len=tmp-vb->name.s;
 						state=F_CR;
 						saved_state=L_VER;
 						break;
 					case FIN_VER:
-						*tmp=0;
 						vb->version.len=tmp-vb->version.s;
 						state=F_CR;
 						saved_state=L_PROTO;
@@ -998,12 +1116,10 @@ parse_again:
 			case '/':
 				switch(state){
 					case FIN_SIP:
-						*tmp=0;
 						vb->name.len=tmp-vb->name.s;
 						state=F_VER;
 						break;
 					case FIN_VER:
-						*tmp=0;
 						vb->version.len=tmp-vb->version.s;
 						state=F_PROTO;
 						break;
@@ -1027,12 +1143,8 @@ parse_again:
 						state=SIP1;
 						vb->name.s=tmp;
 						break;
-					/* allow S in PROTO */
-					case F_PROTO:
-						state=P_PROTO;
-						vb->transport.s=tmp;
-						break;
-					case P_PROTO:
+					case TLS2:
+						state=FIN_TLS;
 						break;
 					default:
 						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
@@ -1046,13 +1158,6 @@ parse_again:
 					case SIP1:
 						state=SIP2;
 						break;
-					/* allow i in PROTO */
-					case F_PROTO:
-						vb->transport.s=tmp;
-						state=P_PROTO;
-						break;
-					case P_PROTO:
-						break;
 					default:
 						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
 								" state %d\n", *tmp, state);
@@ -1066,11 +1171,73 @@ parse_again:
 						state=FIN_SIP;
 						break;
 					/* allow p in PROTO */
+					case UDP2:
+						state=FIN_UDP;
+						break;
+					case TCP2:
+						state=FIN_TCP;
+						break;
+					default:
+						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
+								" state %d\n", *tmp, state);
+						goto error;
+				}
+				break;
+			case 'U':
+			case 'u':
+				switch(state){
 					case F_PROTO:
-						state=P_PROTO;
+						state=UDP1;
 						vb->transport.s=tmp;
 						break;
-					case P_PROTO:
+					default:
+						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
+								" state %d\n", *tmp, state);
+						goto error;
+				}
+				break;
+			case 'D':
+			case 'd':
+				switch(state){
+					case UDP1:
+						state=UDP2;
+						break;
+					default:
+						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
+								" state %d\n", *tmp, state);
+						goto error;
+				}
+				break;
+			case 'T':
+			case 't':
+				switch(state){
+					case F_PROTO:
+						state=TCP_TLS1;
+						vb->transport.s=tmp;
+						break;
+					default:
+						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
+								" state %d\n", *tmp, state);
+						goto error;
+				}
+				break;
+			case 'C':
+			case 'c':
+				switch(state){
+					case TCP_TLS1:
+						state=TCP2;
+						break;
+					default:
+						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
+								" state %d\n", *tmp, state);
+						goto error;
+				}
+				break;
+			case 'L':
+			case 'l':
+				switch(state){
+					case TCP_TLS1:
+						state=TLS2;
 						break;
 					default:
 						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
@@ -1085,13 +1252,6 @@ parse_again:
 						state=VER1;
 						vb->version.s=tmp;
 						break;
-					/* allow 2 in PROTO*/
-					case F_PROTO:
-						vb->transport.s=tmp;
-						state=P_PROTO;
-						break;
-					case P_PROTO:
-						break;
 					default:
 						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
 								" state %d\n", *tmp, state);
@@ -1103,13 +1263,6 @@ parse_again:
 					case VER1:
 						state=VER2;
 						break;
-					/* allow . in PROTO */
-					case F_PROTO:
-						vb->transport.s=tmp;
-						state=P_PROTO;
-						break;
-					case P_PROTO:
-						break;
 					default:
 						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
 								" state %d\n", *tmp, state);
@@ -1121,13 +1274,6 @@ parse_again:
 					case VER2:
 						state=FIN_VER;
 						break;
-					/* allow 0 in PROTO*/
-					case F_PROTO:
-						vb->transport.s=tmp;
-						state=P_PROTO;
-						break;
-					case P_PROTO:
-						break;
 					default:
 						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
 								" state %d\n", *tmp, state);
@@ -1136,18 +1282,9 @@ parse_again:
 				break;
 			
 			default:
-				switch(state){
-					case F_PROTO:
-						vb->transport.s=tmp;
-						state=P_PROTO;
-						break;
-					case P_PROTO:
-						break;
-					default:
 						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
 								" state %d\n", *tmp, state);
 						goto error;
-				}
 				break;
 		}
 	} /* for tmp*/
@@ -1170,7 +1307,7 @@ parse_again:
 					case F_HOST:/*eat the spaces*/
 						break;
 					case P_HOST:
-						 *tmp=0;/*mark end of host*/
+						 /*mark end of host*/
 						 vb->host.len=tmp-vb->host.s;
 						 state=L_PORT;
 						 break;
@@ -1178,7 +1315,7 @@ parse_again:
 					case F_PORT:
 						break;
 					case P_PORT:
-						*tmp=0; /*end of port */
+						/*end of port */
 						vb->port_str.len=tmp-vb->port_str.s;
 						state=L_PARAM;
 						break;
@@ -1195,13 +1332,10 @@ parse_again:
 					case F_COMMENT:
 					case P_COMMENT:
 						break;
-					case F_IP6HOST: /*eat the spaces*/
-						break;
+					case F_IP6HOST: /*no spaces allowed*/
 					case P_IP6HOST:
-						*tmp=0; /*mark end of host*/
-						vb->host.len=tmp-vb->host.s;
-						state=L_PORT; 
-						break;
+						LOG(L_ERR, "ERROR:parse_via: bad ipv6 reference\n");
+						goto error;
 					case F_CRLF:
 					case F_LF:
 					case F_CR:
@@ -1231,13 +1365,13 @@ parse_again:
 						state=F_LF;
 						break;
 					case P_HOST:
-						 *tmp=0;/*mark end of host*/
+						 /*mark end of host*/
 						 vb->host.len=tmp-vb->host.s;
 						 saved_state=L_PORT;
 						 state=F_LF;
 						 break;
 					case P_PORT:
-						*tmp=0; /*end of port */
+						/*end of port */
 						vb->port_str.len=tmp-vb->port_str.s;
 						saved_state=L_PARAM;
 						state=F_LF;
@@ -1277,13 +1411,13 @@ parse_again:
 						state=F_CR;
 						break;
 					case P_HOST:
-						 *tmp=0;/*mark end of host*/
+						 /*mark end of host*/
 						 vb->host.len=tmp-vb->host.s;
 						 saved_state=L_PORT;
 						 state=F_CR;
 						 break;
 					case P_PORT:
-						*tmp=0; /*end of port */
+						/*end of port */
 						vb->port_str.len=tmp-vb->port_str.s;
 						saved_state=L_PARAM;
 						state=F_CR;
@@ -1309,13 +1443,12 @@ parse_again:
 				switch(state){
 					case F_HOST:
 					case F_IP6HOST:
-						LOG(L_ERR,"ERROR:parse_via:"
-							" no host found\n");
-						goto error;
+						state=P_IP6HOST;
+						break;
 					case P_IP6HOST:
 						break;
 					case P_HOST:
-						*tmp=0; /*mark  end of host*/
+						/*mark  end of host*/
 						vb->host.len=tmp-vb->host.s;
 						state=F_PORT;
 						break;
@@ -1367,16 +1500,17 @@ parse_again:
 						LOG(L_ERR, "ERROR:parse_via: bad ipv6 reference\n");
 						goto error;
 					case P_HOST:
-						*tmp=0;
 						vb->host.len=tmp-vb->host.s;
 						state=F_PARAM;
+						param_start=tmp+1;
 						break;
 					case P_PORT:
-						*tmp=0; /*mark the end*/
+						/*mark the end*/
 						vb->port_str.len=tmp-vb->port_str.s;
 					case L_PORT:
 					case L_PARAM:
 						state=F_PARAM;
+						param_start=tmp+1;
 						break;
 					case F_PORT:
 						LOG(L_ERR, "ERROR:parse_via:"
@@ -1390,6 +1524,7 @@ parse_again:
 					case P_PARAM:
 						/*hmm next, param?*/
 						state=F_PARAM;
+						param_start=tmp+1;
 						break;
 					case L_VIA:
 					case F_VIA:
@@ -1427,12 +1562,12 @@ parse_again:
 						LOG(L_ERR, "ERROR:parse_via: bad ipv6 reference\n");
 						goto error;
 					case P_HOST:
-						*tmp=0; /*mark the end*/
+						/*mark the end*/
 						vb->host.len=tmp-vb->host.s;
 						state=F_VIA;
 						break;
 					case P_PORT:
-						*tmp=0; /*mark the end*/
+						/*mark the end*/
 						vb->port_str.len=tmp-vb->port_str.s;
 						state=F_VIA;
 						break;
@@ -1482,19 +1617,19 @@ parse_again:
 							*tmp, state);
 						goto  error;
 					case P_HOST:
-						*tmp=0; /*mark the end*/
+						/*mark the end*/
 						vb->host.len=tmp-vb->host.s;
 						state=F_COMMENT;
 						c_nest++;
 						break;
 					case P_PORT:
-						*tmp=0; /*mark the end*/
+						/*mark the end*/
 						vb->port_str.len=tmp-vb->port_str.s;
 						state=F_COMMENT;
 						c_nest++;
 						break;
 					case P_PARAM:
-						*tmp=0; /*mark the end*/
+						/*mark the end*/
 						vb->params.len=tmp-vb->params.s;
 						state=F_COMMENT;
 						c_nest++;
@@ -1505,7 +1640,6 @@ parse_again:
 						state=F_COMMENT;
 						vb->params.len=tmp-vb->params.s;
 						c_nest++;
-						*tmp=0;
 						break;
 					case P_COMMENT:
 					case F_COMMENT:
@@ -1531,7 +1665,6 @@ parse_again:
 							c_nest--;
 							if(c_nest==0){
 								state=L_VIA;
-								*tmp=0;
 								vb->comment.len=tmp-vb->comment.s;
 								break;
 							}
@@ -1575,6 +1708,7 @@ parse_again:
 			case '[':
 				switch(state){
 					case F_HOST:
+						vb->host.s=tmp; /* mark start here (include [])*/
 						state=F_IP6HOST;
 						break;
 					case F_COMMENT:/*everything is allowed in a comment*/
@@ -1598,8 +1732,8 @@ parse_again:
 			case ']':
 				switch(state){
 					case P_IP6HOST:
-						*tmp=0; /*mark the end*/
-						vb->host.len=tmp-vb->host.s;
+						/*mark the end*/
+						vb->host.len=(tmp-vb->host.s)+1; /* include "]" */
 						state=L_PORT;
 						break;
 					case F_CRLF:
@@ -1638,7 +1772,7 @@ parse_again:
 						break;
 					case F_PARAM:
 						/*state=P_PARAM*/;
-						if(vb->params.s==0) vb->params.s=tmp;
+						if(vb->params.s==0) vb->params.s=param_start;
 						param=pkg_malloc(sizeof(struct via_param));
 						if (param==0){
 							LOG(L_ERR, "ERROR:parse_via: mem. allocation"
@@ -1646,27 +1780,30 @@ parse_again:
 							goto error;
 						}
 						memset(param,0, sizeof(struct via_param));
+						param->start=param_start;
 						tmp=parse_via_param(tmp, end, &state, &saved_state,
 											param);
 
 						switch(state){
-							case L_PARAM:
 							case F_PARAM:
+								param_start=tmp+1;
+							case L_PARAM:
 							case F_LF:
 							case F_CR:
 								break;
 							case F_VIA:
-								vb->params.len=tmp-vb->params.s;
-								*tmp=0;
+								vb->params.len=param->start+param->size
+												-vb->params.s;
 								break;
 							case END_OF_HEADER:
-								vb->params.len=tmp-vb->params.s;
-								state=saved_state;
-								goto endofheader;
+								vb->params.len=param->start+param->size
+												-vb->params.s;
+								break;
 							case PARAM_ERROR:
 								pkg_free(param);
 								goto error;
 							default:
+								pkg_free(param);
 								LOG(L_ERR, "ERROR: parse_via after"
 										" parse_via_param: invalid"
 										" char <%c> on state %d\n",
@@ -1681,6 +1818,15 @@ parse_again:
 							vb->branch=param;
 						else if (param->type==PARAM_RECEIVED)
 							vb->received=param;
+						else if (param->type==PARAM_RPORT)
+							vb->rport=param;
+						else if (param->type==PARAM_I)
+							vb->i=param;
+						
+						if (state==END_OF_HEADER){
+							state=saved_state;
+							goto endofheader;
+						}
 						break;
 					case P_PARAM:
 						break;
@@ -1702,7 +1848,6 @@ parse_again:
 						break;
 					case F_IP6HOST:
 						state=P_IP6HOST;
-						vb->host.s=tmp;
 						break;
 					case P_IP6HOST:
 						break;
@@ -1762,10 +1907,10 @@ endofpacket:
 	vb->error=PARSE_OK;
 	vb->bsize=tmp-buffer;
 	if (vb->port_str.s){
-		vb->port=str2s((unsigned char*)vb->port_str.s, vb->port_str.len, &err);
+		vb->port=str2s(vb->port_str.s, vb->port_str.len, &err);
 		if (err){
-					LOG(L_ERR, "ERROR: parse_via: invalid port number <%s>\n",
-						vb->port_str.s);
+					LOG(L_ERR, "ERROR: parse_via: invalid port number <%.*s>\n",
+						vb->port_str.len, ZSW(vb->port_str.s));
 					goto error;
 		}
 	}
@@ -1775,10 +1920,10 @@ nextvia:
 	vb->error=PARSE_OK;
 	vb->bsize=tmp-buffer;
 	if (vb->port_str.s){
-		vb->port=str2s((unsigned char*)vb->port_str.s, vb->port_str.len, &err);
+		vb->port=str2s(vb->port_str.s, vb->port_str.len, &err);
 		if (err){
-					LOG(L_ERR, "ERROR: parse_via: invalid port number <%s>\n",
-						vb->port_str.s);
+					LOG(L_ERR, "ERROR: parse_via: invalid port number <%.*s>\n",
+						vb->port_str.len, ZSW(vb->port_str.s));
 					goto error;
 		}
 	}
@@ -1794,11 +1939,11 @@ nextvia:
 
 error:
 	if (end>buffer){
-		LOG(L_ERR, "ERROR: parse_via on: <%.*s>\n", end-buffer, buffer);
+		LOG(L_ERR, "ERROR: parse_via on: <%.*s>\n", (int)(end-buffer), ZSW(buffer));
 	}
 	if ((tmp>buffer)&&(tmp<end)){
 		LOG(L_ERR, "ERROR: parse_via parse error, parsed so far:<%.*s>\n",
-				tmp-buffer, buffer );
+				(int)(tmp-buffer), ZSW(buffer) );
 	}else{
 		LOG(L_ERR, "ERROR: parse_via: via parse error\n");
 	}
