@@ -1,5 +1,5 @@
 /*
- * $Id: receive.h,v 1.4 2002/09/19 12:23:52 jku Rel $
+ * $Id: receive.h,v 1.5 2002/12/12 21:46:38 andrei Exp $
  *
  * Copyright (C) 2001-2003 Fhg Fokus
  *
@@ -32,7 +32,7 @@
 
 #include "ip_addr.h"
 
-int receive_msg(char* buf, unsigned int len, union sockaddr_union *src_su);
+int receive_msg(char* buf, unsigned int len, struct receive_info *ri);
 
 
 #endif
